MatMut

Struct MatMut 

pub struct MatMut<'a, T> { /* private fields */ }
mutable matrix type column major

Implementations

Source

impl<'a, T> MatMut<'a, T>

pub fn new(buffer: &'a mut [T], dimension: (usize, usize)) -> Self

constructs MatMut with given slice and (n_rows, n_cols) dimension

example:

use lak::types::MatMut; 

// col-major 2 x 3 matrix: 
// [1 3 5] 
// [2 4 6] 
let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]; 
let a = MatMut::new(&mut a, (2, 3));
pub fn as_slice(&self) -> &[T]

accesses full internal immutable slice

pub fn as_slice_mut(&mut self) -> &mut [T]

accesses full internal slice as mutable

pub fn dimension(&self) -> (usize, usize)

accesses matrix dimension (n_rows, n_cols)

pub fn n_rows(&self) -> usize

accesses matrix number of rows

pub fn n_cols(&self) -> usize

accesses matrix number of cols

pub fn col(&self, j: usize) -> VecRef<'_, T>

return a VecRef for a given column in Self

pub fn col_mut(&mut self, j: usize) -> VecMut<'_, T>

return a VecMut for a given column in Self

pub fn col_panel(&self, cols: Range<usize>) -> MatRef<'_, T>

return a MatRef for a contiguous column panel of Self

contains full columns over a given a range of column indices.

example:

use lak::types::{MatRef, MatMut}; 

// [1 3 5] 
// [2 4 6] 
let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]; 
let a = MatMut::new(&mut a, (2, 3)); 

// MatRef of columns 1..3 
// [3 5] 
// [4 6] 
let panel = a.col_panel(1..3); 
pub fn col_panel_mut(&mut self, cols: Range<usize>) -> MatMut<'_, T>

returns a MatMut for a contiguous column panel of Self

contains full columns over a given a range of column indices.

example:

use lak::types::MatMut; 

// [1 3 5] 
// [2 4 6] 
let mut a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]; 
let mut a = MatMut::new(&mut a, (2, 3)); 

// MatMut of columns 1..3 
// [3 5] 
// [4 6] 
let panel = a.col_panel_mut(1..3); 
pub fn col_panels( &self, nc: usize, ) -> impl DoubleEndedIterator<Item = (Range<usize>, MatRef<'_, T>)>

return an Iterator over MatRefs chunks containing column panels that span Self.

each chunk holds nc columns, and the last item is the leftover column panel with n_cols < nc

args:

  • nc: usize - # cols in panel

returns:

pub fn has_len_equal_to_n_cols(&self, length: usize) -> bool

checks whether matrix n_cols equals given length

pub fn has_len_equal_to_n_rows(&self, length: usize) -> bool

checks whether matrix n_rows equals given length

pub fn reborrow(&mut self) -> MatMut<'_, T>

used for calling routines over and over again on the stored internal mutable slice

borrows self mutably

example:

use lak::l2::ger;
use lak::types::{MatMut, VecRef};

let x = [1.0, 2.0];
let y = [3.0, 4.0];
let mut a = [0.0; 4];

let x = VecRef::new(&x);
let y = VecRef::new(&y);
let mut a = MatMut::new(&mut a, (2, 2));

ger(1.0, a.reborrow(), x, y);
ger(1.0, a.reborrow(), x, y);