MatRef

Struct MatRef 

pub struct MatRef<'a, T> { /* private fields */ }
immutable matrix type column major

Implementations

Source

impl<'a, T> MatRef<'a, T>

pub fn new(buffer: &'a [T], dimension: (usize, usize)) -> Self

constructs MatRef with given slice and (n_rows, n_cols) dimension

example:

use lak::types::MatRef; 

// col-major 2 x 3 matrix: 
// [1 3 5] 
// [2 4 6] 
let a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]; 
let a = MatRef::new(&a, (2, 3));
pub fn as_slice(&self) -> &[T]

accesses internal immutable slice

pub fn dimension(&self) -> (usize, usize)

accesses matrix dimension (n_rows, n_cols)

pub fn n_rows(&self) -> usize

accesses matrix number of rows

pub fn n_cols(&self) -> usize

accesses matrix number of cols

pub fn col(&self, j: usize) -> VecRef<'_, T>

return a VecRef for a given column in Self

pub fn col_panel(&self, cols: Range<usize>) -> MatRef<'_, T>

return a MatRef for a contiguous column panel of Self

example:

use lak::types::MatRef; 

// [1 3 5] 
// [2 4 6] 
let a = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]; 
let a = MatRef::new(&a, (2, 3)); 

// MatRef of columns 1..3 
// [3 5] 
// [4 6] 
let panel = a.col_panel(1..3); 
pub fn col_panels( &self, nc: usize, ) -> impl DoubleEndedIterator<Item = (Range<usize>, MatRef<'_, T>)>

returns an Iterator over MatRefs containing column panels that span Self.

each panel holds nc columns, and the last item is the leftover panel with column width < nc

args:

  • nc: usize - # cols in panel

returns:

pub fn has_len_equal_to_n_cols(&self, length: usize) -> bool

checks whether matrix n_cols equals given length

pub fn has_len_equal_to_n_rows(&self, length: usize) -> bool

checks whether matrix n_rows equals given length