VecMut

Struct VecMut 

pub struct VecMut<'a, T> { /* private fields */ }
mutable vector type

Implementations

Source

impl<'a, T> VecMut<'a, T>

pub fn new(buffer: &'a mut [T]) -> Self

constructs VecMut with given slice

pub fn length(&self) -> usize

returns length of internal slice

pub fn as_slice(&self) -> &[T]

accesses full internal slice as immutable

pub fn slice(&self, range: Range<usize>) -> &[T]

accesses internal immutable slice over a given index range

pub fn slice_mut(&mut self, range: Range<usize>) -> &mut [T]

accesses internal mutable slice over a given index range

pub fn as_slice_mut(&mut self) -> &mut [T]

accesses full internal slice as mutable

pub fn has_equal_length(&self, length: usize) -> bool

checks whether internal length is equal to given length parameter

pub fn reborrow(&mut self) -> VecMut<'_, T>

used for calling routines over and over again on the stored internal mutable slice

borrows self mutably

example:

use lak::l1::scal;
use lak::types::VecMut;

let mut x = [1.0, 2.0, 3.0];
let mut x = VecMut::new(&mut x);

scal(2.0, x.reborrow());
scal(3.0, x.reborrow());