pub struct VecMut<'a, T> { /* private fields */ }Expand description
mutable vector type
Implementations
Sourceimpl<'a, T> VecMut<'a, T>
impl<'a, T> VecMut<'a, T>
Sourcepub fn slice(&self, range: Range<usize>) -> &[T]
pub fn slice(&self, range: Range<usize>) -> &[T]
accesses internal immutable slice over a given index range
Sourcepub fn slice_mut(&mut self, range: Range<usize>) -> &mut [T]
pub fn slice_mut(&mut self, range: Range<usize>) -> &mut [T]
accesses internal mutable slice over a given index range
Sourcepub fn as_slice_mut(&mut self) -> &mut [T]
pub fn as_slice_mut(&mut self) -> &mut [T]
accesses full internal slice as mutable
Sourcepub fn has_equal_length(&self, length: usize) -> bool
pub fn has_equal_length(&self, length: usize) -> bool
checks whether internal length is equal to given length parameter
Sourcepub fn reborrow(&mut self) -> VecMut<'_, T>
pub fn reborrow(&mut self) -> VecMut<'_, T>
used for calling routines over and over again on the stored internal mutable slice
borrows self mutably
example:
use lak::l1::scal;
use lak::types::VecMut;
let mut x = [1.0, 2.0, 3.0];
let mut x = VecMut::new(&mut x);
scal(2.0, x.reborrow());
scal(3.0, x.reborrow());Trait Implementations